Package: SmoothPLS
Type: Package
Title: Partial Least-Squares Algorithm for Categorical and Scalar
        Functional Data
Version: 0.1.5
Date: 2026-04-10
Authors@R: c(
    person(
      "Francois", "Bassac",
      email = "fr.bassac@gmail.com",
      role = c("aut", "cre")
    )
  )
Description: Performs the Partial Least-Squares ('PLS') algorithm for
    functional data through the concept of active area integration. This 
    approach builds upon the basis expansion methods for functional 'PLS' 
    regression described in Aguilera et al. (2010) 
    <doi:10.1016/j.chemolab.2010.09.007>. The package seamlessly handles both 
    Scalar Functional Data ('SFD') and Categorical Functional Data ('CFD'), 
    providing interpretable regression curves even for discrete 
    state changes. It was developed during a PhD thesis between 'DECATHLON' 
    and French research institute 'INRIA' 2022-2026. The 'SmoothPLS' method does not directly
    decompose the data into a basis; rather, it assumes the data is known as 
    precisely as desired, and for every 'PLS' component, the weight functions are 
    decomposed into the basis. For both single-state and multi-state 'CFD' 
    as well as 'SFD', the algorithm is implemented for a scalar response. 
    To provide a baseline, a naive 'PLS' method on time-value functions and 
    standard Functional 'PLS' are also implemented.
License: MIT + file LICENSE
URL: https://github.com/FrancoisBassac/SmoothPLS,
        https://FrancoisBassac.github.io/SmoothPLS/
BugReports: https://github.com/FrancoisBassac/SmoothPLS/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Imports: cfda (>= 0.12.1), dplyr (>= 1.1.4), fda (>= 6.2.0), ggplot2
        (>= 3.5.1), MASS (>= 7.3-64), mgcv (>= 1.9-1), pls (>= 2.8-5),
        pracma (>= 2.4.4), stats (>= 4.4.1), tidyr (>= 1.3.1), utils
        (>= 4.4.1), rlang (>= 1.1.0), magrittr (>= 2.0.3),
        future.apply, future
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-29 17:11:27 UTC; Francois
Author: Francois Bassac [aut, cre]
Maintainer: Francois Bassac <fr.bassac@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 19:20:02 UTC
Built: R 4.6.0; ; 2026-05-04 22:12:16 UTC; unix
