# Generated by roxygen2: do not edit by hand

export(assemble_basis_metric)
export(assert_multivariate_smoothPLS_inputs)
export(beta_1_real_func)
export(beta_2_real_func)
export(beta_3_real_func)
export(beta_4_real_func)
export(beta_5_real_func)
export(beta_6_real_func)
export(beta_7_real_func)
export(beta_list_generation)
export(block_diag)
export(build_df_per_state)
export(build_reg_curve_spls)
export(build_spls_functions)
export(cat_data_to_indicator)
export(convert_to_wide_format)
export(create_bspline_basis)
export(determine_next_state)
export(eval_max_min_y)
export(evaluate_V_i_function)
export(evaluate_curves_distances)
export(evaluate_gamma_ij)
export(evaluate_id_func_integral)
export(evaluate_id_func_integral_deprecated)
export(evaluate_lambda)
export(evaluate_metric)
export(evaluate_reg_curve_SPLS_uni)
export(evaluate_results)
export(evaluate_variance_explained)
export(from_basis_to_fdlist)
export(from_fd_to_func)
export(funcPLS)
export(funcPLS_predict)
export(generate_X_df)
export(generate_X_df_CFD)
export(generate_X_df_SFD)
export(generate_X_df_SFD_data)
export(generate_X_df_multistates)
export(generate_X_df_test)
export(generate_Y_df)
export(generate_Y_df_CFD)
export(generate_probabilities)
export(gram_schmidt_orthonormalize)
export(help_smoothPLS)
export(initial_state_determination)
export(is_orthogonal)
export(is_orthonormal)
export(lambda_determination)
export(mae_values)
export(multivariate_assemble_basis_metric)
export(naivePLS)
export(naivePLS_formatting)
export(naivePLS_predict)
export(number_of_test_id)
export(obj_list_creation)
export(orthonormalize_basis_list)
export(plot_CFD_individuals)
export(plot_fd_list)
export(plot_model_metrics_base)
export(plot_real_and_smoothed_data_ind)
export(press_model)
export(press_values)
export(r_squared_values)
export(reg_curve_funcPLS_evaluation)
export(regularize_time_series)
export(remove_duplicate_states)
export(select_from_fd_list)
export(smoothPLS)
export(smoothPLS_predict)
export(smoothPLS_predict_uni)
export(split_in_state_df)
export(state_indicator)
export(state_indicator_old)
export(test_basis_properties)
export(transfer_probabilities)
export(transition_matrix)
import(ggplot2)
import(graphics)
import(pls)
importFrom(MASS,ginv)
importFrom(cfda,plotData)
importFrom(dplyr,full_join)
importFrom(fda,Data2fd)
importFrom(fda,create.bspline.basis)
importFrom(fda,eval.fd)
importFrom(fda,fd)
importFrom(fda,inprod)
importFrom(fda,norder.bspline)
importFrom(future,availableCores)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future,sequential)
importFrom(future.apply,future_lapply)
importFrom(future.apply,future_sapply)
importFrom(graphics,plot)
importFrom(graphics,title)
importFrom(magrittr,"%>%")
importFrom(mgcv,mroot)
importFrom(pls,MSEP)
importFrom(pls,RMSEP)
importFrom(pls,plsr)
importFrom(pracma,trapz)
importFrom(rlang,.data)
importFrom(stats,approx)
importFrom(stats,approxfun)
importFrom(stats,coef)
importFrom(stats,integrate)
importFrom(stats,lm.influence)
importFrom(stats,plogis)
importFrom(stats,predict)
importFrom(stats,rbinom)
importFrom(stats,residuals)
importFrom(stats,rexp)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(tidyr,all_of)
importFrom(tidyr,pivot_wider)
importFrom(utils,tail)
